/**
  *  <p>WHY ARE ____ PEOPLE ____ ? : Perceptions of the world according to Google Suggest&trade;<br />
  *  by Nara Kasbergen (nara@nara-designs.com)</p>
  *
  *  <p>This project seeks to explore our stereotypes and perceptions of the people of our own country and other countries using the Google Suggest API.<br />
  *  It searches Google Suggest for queries such as <i>"why are americans"</i> and <i>"why are british people"</i> and records the most common adjectives and phrases.<br />
  *  It then displays the attributes associated with each country in a <a href="http://www.personaldna.com" target="_blank">PersonalDNA</a>-like strip, color-coded
  *  according to the positive, negative, <br />or neutral connotation of the phrase. Positive is green, negative is red, and neutral is blue.</p>
  *
  *  <p>The user can:<ul>
  *  <li>Hover over any part of the "DNA strip" to uncover words that do not fit (ie. the label is wider than the part of the strip).</li>
  *  <li>Click on a word or phrase and see all the countries that have that trait, ranked by the percentage of their search results that that phrase takes up.</li>
  *  <li>Click on one of the tabs on the bottom to examine how the attributes change when different geographic localizations of Google are queried.</li>
  *  <li>Click on the name of a country to dim out the other countries. This setting is retained when switching tabs at the bottom, making it easier to <br />
  *      examine the changes in a specific country across the different geographic localizations of Google.</li></ul>
  *  More features might be added at a later date.</p>
  *
  */

boolean scrapeOn = false;
PFont defaultFont;
PFont bigFont;
PFont pixelFont;
PImage googleLogo;
String countryFile = "countries.csv";
String adjectivesFile = "adjectives.csv";
String sourcesFile = "sources.txt";
HashMap countryList = new HashMap();
HashMap attributesBySource = new HashMap();
CountryDisplay[] displayCountries = new CountryDisplay[0];
HashMap adjectivePercentages = new HashMap();
HashMap allAdjectives = new HashMap();
HashMap adjectiveColorIndexes = new HashMap();
color[] adjectiveColors = new color[0];
int dimModeIndex = -1;
String selectedWord;
boolean wordSortMode = false;
Sources sourcesDisplay;
String selectedSource;

void setup()
{
  size(1200, 920);
  bigFont = loadFont("QlassikBold-36.vlw");
  defaultFont = loadFont("QlassikMedium-24.vlw");
  pixelFont = loadFont("FFFHarmony-8.vlw");
  textFont(defaultFont, 24);
  textAlign(CENTER, CENTER);
  fill(#000000);
  googleLogo = loadImage("google.png");
  
  String[] adjsList = loadStrings(adjectivesFile);
  for (int i = 0; i < adjsList.length; i++)
  {
    String[] lineParts = adjsList[i].split(",");
    if (lineParts.length > 1) { allAdjectives.put(lineParts[0], lineParts[1]); }
  }
  
  sourcesDisplay = new Sources();
  selectedSource = sourcesDisplay.sources[0].name;
  
  /* Creating our search strings and scraping the data */
  HashMap countrySearchStrings = new HashMap();
  int numSearchStrings = 2;
  String[] countryCSV = loadStrings(countryFile);
  for (int n = 0; n < countryCSV.length; n++)
  {
    String currLine = countryCSV[n];
    String[] lineParts = split(currLine, ",");
    String country = lineParts[0];
    countryList.put(lineParts[0], lineParts[1]);
    
    String[] searchStrings = new String[numSearchStrings];
    searchStrings[0] = "why are ";
    String[] matches = match(lineParts[1], ".+(n|i|k)$");
    if (matches != null) { searchStrings[0] += lineParts[1].toLowerCase() + "s"; }
    else { searchStrings[0] += "the " + lineParts[1].toLowerCase(); }
    searchStrings[1] = "why are " + lineParts[1].toLowerCase() + " people";
    
    if (scrapeOn) { scrapeData(country, searchStrings); }
    countrySearchStrings.put(country, searchStrings);
  }
  
  /* Parsing the saved XML files into a hierarchical tree structure */
  for (int k = 0; k < sourcesDisplay.sources.length; k++)
  {    
    HashMap attributesByCountry = new HashMap();
    Object[] countries = countrySearchStrings.keySet().toArray();
    for (int n = 0; n < countries.length; n++)
    {
      String country = countries[n].toString();
      String[] searchStrings = (String[]) countrySearchStrings.get(country);
      HashMap adjectives = new HashMap();
      
      for (int j = 0; j < numSearchStrings; j++)
      {
        String[] matches = match(sourcesDisplay.sources[k].name, "([a-z]+)$");
        String suffix = matches[1];
        String myFileName = join(split(country, " "), "_").toLowerCase();
        if (k != 0) { myFileName += "_" + suffix; }
        myFileName += "_" + j;
        XMLElement xml = new XMLElement(this, myFileName + ".xml");
        int numResults = xml.getChildCount();
        for (int i = 0; i < numResults; i++)
        {
          XMLElement result = xml.getChild(i);
          String query = ((XMLElement) result.getChild(0)).getStringAttribute("data");
          if (match(query, searchStrings[j]) == null) { continue; }
          query = trim(query.substring(searchStrings[j].length()));
          XMLElement numQueries = result.getChild(1);
          int num_queries = numQueries.getIntAttribute("int");
          
          String[] m1 = match(query, "^(called|referred|and |'s|in |eyes|steps|army|fighting|coming|[Ff]rench|not allowed|against|[a-zA-Z]+ing )"); if (m1 != null) { continue; }
          String[] m2 = match(query, " (song|video)$"); if (m2 != null) { continue; }
          String[] m3 = match(query, "^s?o (.+)$"); if (m3 != null) { query = trim(m3[1]); }
          
          if (adjectives.containsKey(query)) { adjectives.put(query, ((Integer) adjectives.get(query)) + num_queries); } else { adjectives.put(query, num_queries); }
          
          if (!allAdjectives.containsKey(query)) { allAdjectives.put(query, "unknown"); }
          if (!adjectiveColorIndexes.containsKey(query))
          {
            color myColor = color(random(100, 225), random(100, 225), random(100, 225));
            String property = (String) allAdjectives.get(query);
            if (match(property, "positive") != null) { myColor = color(random(30, 100), random(150, 255), random(30, 100)); }
            if (match(property, "negative") != null) { myColor = color(random(150, 255), random(30, 100), random(30, 100)); }
            if (match(property, "neutral")  != null) { myColor = color(random(30, 100), random(30, 100), random(150, 255)); }
            adjectiveColors = (color[]) append(adjectiveColors, myColor);
            adjectiveColorIndexes.put(query, adjectiveColors.length - 1);
          }
        }
      }
      attributesByCountry.put(countryList.get(country), adjectives);
    }
    attributesBySource.put(sourcesDisplay.sources[k].name, attributesByCountry);
  }
  
  createNewObjects();
  
  /* Saving all of our found adjectives to an external file */
  if (!online)
  {
    PrintWriter adjListFile = createWriter("data/" + adjectivesFile);
    Object[] adjs = allAdjectives.keySet().toArray();
    for (int i = 0; i < allAdjectives.size(); i++)
    {
      String adjective = adjs[i].toString();
      String value = (String) allAdjectives.get(adjective);
      adjListFile.println(adjective + "," + value);
    }
    adjListFile.flush();
    adjListFile.close();
  }
  
  smooth();
  noStroke();
  noLoop();
}

void scrapeData(String country, String[] searchStrings)
{
  for (int n = 0; n < sourcesDisplay.sources.length; n++)
  {
    String[] matches = match(sourcesDisplay.sources[n].name, "([a-z]+)$");
    String suffix = matches[1];
    for (int i = 0; i < searchStrings.length; i++)
    {
      String myFileName = join(split(country, " "), "_").toLowerCase();
      if (n != 0) { myFileName += "_" + suffix; }
      myFileName += "_" + i;
      String formattedSearchString = join(split(searchStrings[i], " "), "+");
      saveStrings("data/" + myFileName + ".xml", loadStrings("http://www." + sourcesDisplay.sources[n].name + "/complete/search?output=toolbar&q=" + formattedSearchString));
    }
  }
}

void createNewObjects()
{
  adjectivePercentages = new HashMap();
  displayCountries = new CountryDisplay[0];
  
  HashMap attributesByCountry = (HashMap) attributesBySource.get(selectedSource);
  Object[] keys = attributesByCountry.keySet().toArray();
  String[] countries = new String[keys.length];
  for (int i = 0; i < keys.length; i++) { countries[i] = keys[i].toString(); }
  countries = sort(countries);
  for (int i = 0; i < attributesByCountry.size(); i++)
  {
    String country = countries[i];
    HashMap adjectives = (HashMap) attributesByCountry.get(country);    
    displayCountries = (CountryDisplay[]) append(displayCountries, new CountryDisplay(country, adjectives));
  }
}

void draw()
{
  background(#FFFFFF);
  
  pushStyle();
  textAlign(LEFT, BASELINE);
  textFont(bigFont, 36);
  text("why are ___ people ______ ?", 20, 50);
  textAlign(RIGHT, BASELINE);
  textFont(defaultFont, 24);
  text("Perceptions of the world according to", 940, 50);
  popStyle();
  image(googleLogo, 960, 10);
  
  float hInterval = (height-120)/countryList.size();
  float hOffset = (hInterval/2) + 80;
  
  HashSet displayedCountries = new HashSet();
  if (wordSortMode && (selectedWord != null))
  {
    HashMap thisAdj = (HashMap) adjectivePercentages.get(selectedWord);
    ArrayList countries = new ArrayList(thisAdj.keySet());
    int numCountries = 0;
    while (countries.size() > 0)
    {
      int largestIndex = 0;
      if (countries.size() > 1)
      {
        for (int i = 1; i < countries.size(); i++)
        {
          Float thisPct = (Float) thisAdj.get(countries.get(i));
          if (thisPct > (Float) thisAdj.get(countries.get(largestIndex)))
          {
            largestIndex = i;
          }
        }
      }
      CountryDisplay country = (CountryDisplay) countries.get(largestIndex);
      country.display(hInterval * numCountries + hOffset);
      displayedCountries.add(country.name);
      countries.remove(largestIndex);
      numCountries++;
    }
    for (int i = 0; i < displayCountries.length; i++)
    {
      if (!displayedCountries.contains(displayCountries[i].name))
      {
        displayCountries[i].dim();
        displayCountries[i].display(hInterval * numCountries + hOffset);
        numCountries++;
      }
    }
  }
  else
  {
    for (int i = 0; i < displayCountries.length; i++)
    {
      if (dimModeIndex >= 0 && dimModeIndex != i) { displayCountries[i].dim(); }
      displayCountries[i].display(hInterval * i + hOffset);
    }
  }
  
  sourcesDisplay.display();
  
  for (int i = 0; i < displayCountries.length; i++) { displayCountries[i].undim(); }
  
  //if (!online) { save("nk_project_1_screenshot.png"); }
}

void mouseMoved()
{
  cursor(ARROW);
  for (int i = 0; i < displayCountries.length; i++)
  {
    if (displayCountries[i].hover() != null)
    {
      redraw();
      return;
    }
  }
  if (!wordSortMode) 
  {
    for (int i = 0; i < displayCountries.length; i++)
    {
      if (displayCountries[i].nameHover())
      {
        redraw();
        return;
      }
    }
  }
  for (int i = 0; i < sourcesDisplay.sources.length; i++)
  {
    if (sourcesDisplay.sources[i].hover())
    {
      redraw();
      return;
    }
  }
  redraw();
}

void mouseClicked()
{
  boolean wasWordSortMode = false;
  if (wordSortMode == true) { wasWordSortMode = true; }
  wordSortMode = false;
  for (int i = 0; i < sourcesDisplay.sources.length; i++)
  {
    if (sourcesDisplay.sources[i].hover())
    {
      if (!selectedSource.equals(sourcesDisplay.sources[i].name))
      {
        selectedSource = sourcesDisplay.sources[i].name;
        createNewObjects();
        redraw();
        return;
      }
    }
  }
  dimModeIndex = -1;
  for (int i = 0; i < displayCountries.length; i++)
  {
    selectedWord = displayCountries[i].hover();
    if (selectedWord != null)
    {
      wordSortMode = true;
      redraw();
      return;
    }
  }
  if (!wasWordSortMode)
  {
    for (int i = 0; i < displayCountries.length; i++)
    {
      if (displayCountries[i].nameHover())
      {
        dimModeIndex = i;
        redraw();
        return;
      }
    }
  }
  redraw();
}




class CountryDisplay
{
  float xpos = 200;
  float ypos;
  private int w = 960;
  private int h = 16;
  String name;
  private int totalNum = 0;
  ArrayList attributes = new ArrayList();
  private HashMap myAdjs = new HashMap();
  private boolean dim = false;
  private boolean mouseover = false;
  
  CountryDisplay(String country, HashMap adjectives)
  {
    name = country;
    Object[] adjs = adjectives.keySet().toArray();
    for (int j = 0; j < adjectives.size(); j++)
    {
      String adj = adjs[j].toString();
      int numQueries = (Integer) adjectives.get(adj);
      totalNum += numQueries;
    }
    for (int j = 0; j < adjectives.size(); j++)
    {
      String adj = adjs[j].toString();
      int numQueries = (Integer) adjectives.get(adj);
      float percentage = (float) numQueries/totalNum;
      
      HashMap adjPcts = (HashMap) adjectivePercentages.get(adj);
      if (adjPcts == null) { adjPcts = new HashMap(); }
      adjPcts.put(this, percentage);
      adjectivePercentages.put(adj, adjPcts);
      
      CountryAdjective attribute = new CountryAdjective(percentage, adj);
      attributes.add(attribute);
      myAdjs.put(adj, attribute);
    }
  }
  
  void display(float y)
  {
    ypos = y;
    pushStyle();
    textAlign(RIGHT, CENTER);
    if (dim) { fill(#000000, 80); } else { fill(#000000); }
    text(name, xpos - 20, ypos - 1);
    popStyle();
    
    float lastX = xpos + 0;
    
    if (wordSortMode && selectedWord != null && myAdjs.containsKey(selectedWord))
    {
      int startIndex = attributes.indexOf(myAdjs.get(selectedWord));
      for (int i = startIndex; i < attributes.size(); i++)
      {
        ((CountryAdjective) attributes.get(i)).display(lastX, ypos, w);
        lastX += ((CountryAdjective) attributes.get(i)).percentage*w;
      }
      for (int i = 0; i < startIndex; i++)
      {
        ((CountryAdjective) attributes.get(i)).display(lastX, ypos, w);
        lastX += ((CountryAdjective) attributes.get(i)).percentage*w;
      }
    }
    else
    {
      for (int i = 0; i < attributes.size(); i++)
      {
        ((CountryAdjective) attributes.get(i)).display(lastX, ypos, w);
        lastX += ((CountryAdjective) attributes.get(i)).percentage*w;
      }
    }
  }
  
  String hover()
  {
    int foundIndex = -1;
    for (int i = 0; i < attributes.size(); i++)
    {
      if (!dim) { ((CountryAdjective) attributes.get(i)).undim(); }
      if (((CountryAdjective) attributes.get(i)).hover())
      {
        foundIndex = i;
        break;
      }
    }
    if (foundIndex >= 0)
    {
      for (int i = 0; i < attributes.size(); i++)
      {
        if (i != foundIndex) { ((CountryAdjective) attributes.get(i)).dim(); }
      }
      if (dim) { ((CountryAdjective) attributes.get(foundIndex)).undim(); }
      return ((CountryAdjective) attributes.get(foundIndex)).word;
    }
    
    return null;
  }
  
  boolean nameHover()
  {
    if (mouseX >= 40 && mouseX <= xpos-20 && mouseY >= ypos-(h/2) && mouseY <= ypos+(h/2))
    {
      if (!mouseover) { mouseover = true; }
      cursor(HAND);
      return true;
    }
    else
    {
      if (mouseover) { mouseover = false; }
      return false;
    }
  }
  
  void dim()
  {
    dim = true;
    for (int i = 0; i < attributes.size(); i++) { ((CountryAdjective) attributes.get(i)).dim(); }
  }
  
  void undim()
  {
    dim = false;
    for (int i = 0; i < attributes.size(); i++) { ((CountryAdjective) attributes.get(i)).undim(); }
  }
}

class CountryAdjective
{
  float xpos;
  float ypos;
  float w;
  private int h = 16;
  float percentage;
  String word;
  private color myColor;
  private boolean mouseover = false;
  private boolean dim = false;
  
  CountryAdjective(float pct, String adjective)
  {
    percentage = pct;
    word = adjective;
    int colorIndex = (Integer) adjectiveColorIndexes.get(adjective);
    myColor = adjectiveColors[colorIndex];
  }
  
  void display(float x, float y, float totalW)
  {
    xpos = x;
    ypos = y;
    w = percentage*totalW;
    
    pushStyle();
    if (dim) { fill(myColor, 80); }
    else { fill(myColor, 160); }
    rect(xpos, ypos - (h/2), w, h);
    textFont(pixelFont, 8);
    if (!dim && (textWidth(word) <= w || textWidth(word) - w <= 5 || mouseover))
    {
      fill(#000000);
      text(word, xpos + (w/2), ypos);
    }
    popStyle();
  }
  
  boolean hover()
  {
    if (mouseX >= xpos && mouseX <= (xpos + w) && mouseY >= (ypos - h/2) && mouseY <= (ypos + h/2))
    {
      if (mouseover == false) { mouseover = true; }
      cursor(CROSS);
      return true;
    }
    else
    {
      if (mouseover == true) { mouseover = false; }
      return false;
    }
  }
  
  void dim() { dim = true; }
  void undim() { dim = false; }
}

class Sources
{
  float xpos = 200;
  float ypos = 900;
  float w = 960;
  int h = 20;
  Source[] sources = new Source[0];
  
  Sources()
  {
    String[] srcs = loadStrings(sourcesFile);
    float wInterval = w / srcs.length;
    float lastX = xpos;
    for (int i = 0; i < srcs.length; i++)
    {
      sources = (Source[]) append(sources, new Source(lastX, wInterval, srcs[i]));
      lastX += wInterval;
    }
  }
  
  void display()
  {
    pushStyle();
    textAlign(RIGHT, CENTER);
    textFont(pixelFont, 8);
    text("Source:", xpos - 20, ypos + (h/2));
    popStyle();
    
    for (int i = 0; i < sources.length; i++) { sources[i].display(); }
  }
}

class Source
{
  float xpos;
  float ypos = 900;
  float w;
  int h = 20;
  String name;
  boolean mouseover = false;
  boolean selected = false;
  
  Source(float x, float myW, String src)
  {
    xpos = x;
    w = myW;
    name = src;
  }
  
  void display()
  {
    if (selectedSource.equals(name)) { selected = true; } else { selected = false; }
    
    pushStyle();
    if (selected) { fill(#FFFF00, 160); } else { noFill(); }
    stroke(0);
    if (selected) { rect(xpos, ypos-4, w, h+4); } else { rect(xpos, ypos, w, h); }
    fill(#000000);
    textFont(pixelFont, 8);
    text(name, xpos+(w/2), ypos+(h/2));
    popStyle();
  }
  
  boolean hover()
  {
    if (mouseX >= xpos && mouseX <= (xpos + w) && mouseY >= (ypos - h/2) && mouseY <= (ypos + h/2))
    {
      if (mouseover == false) { mouseover = true; }
      cursor(HAND);
      return true;
    }
    else
    {
      if (mouseover == true) { mouseover = false; }
      return false;
    }
  }
}
